(function() {
  var CouchFunction, DesignDoc, MissingPropPathError, NotAFunctionError, ViewMapFunction, ViewReduceFunction, createSandbox, findModule, inspect, makeRefStack, makeRequireFun, normalizeFunctionDefinition, puts, readPath, runInNewContext, _ref;
  var __slice = Array.prototype.slice, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  };
  runInNewContext = process.binding('evals').Script.runInNewContext;
  _ref = require('sys'), puts = _ref.puts, inspect = _ref.inspect;
  exports.DesignDoc = DesignDoc = function() {
    function DesignDoc(ddoc) {
      this.ddoc = ddoc;
    }
    DesignDoc.prototype.compile = function(funPath, overrides) {
      if (overrides == null) {
        overrides = {};
      }
      return new CouchFunction(this.ddoc, funPath, overrides);
    };
    DesignDoc.prototype.call = function(funPath, funArgs, overrides) {
      var _this;
      if (funArgs == null) {
        funArgs = [];
      }
      if (overrides == null) {
        overrides = {};
      }
      return (_this = this.compile(funPath, overrides)).call.apply(_this, funArgs);
    };
    DesignDoc.prototype.viewMap = function(viewName, funArgs) {
      var fun, retVal;
      if (funArgs == null) {
        funArgs = [];
      }
      fun = new ViewMapFunction(this.ddoc, viewName);
      retVal = fun.call.apply(fun, funArgs);
      return {
        returned: retVal,
        emitted: fun.emitted
      };
    };
    DesignDoc.prototype.viewReduce = function(viewName, funArgs) {
      var fun;
      if (funArgs == null) {
        funArgs = [];
      }
      fun = new ViewReduceFunction(this.ddoc, viewName);
      return fun.call.apply(fun, funArgs);
    };
    DesignDoc.prototype.require = function(moduleID) {
      var code, modulePath, sandbox;
      modulePath = moduleID.replace('/', '.');
      code = readPath(modulePath, this.ddoc);
      sandbox = createSandbox(this.ddoc, modulePath, [], {
        exports: {}
      });
      runInNewContext(code, sandbox, "" + this.ddoc._id + "/" + moduleID);
      return sandbox.exports;
    };
    return DesignDoc;
  }();
  CouchFunction = function() {
    function CouchFunction(ddoc, funPath, overrides) {
      var code, sandbox;
      this.ddoc = ddoc;
      this.funPath = funPath;
      this.overrides = overrides != null ? overrides : {};
      this.log = [];
      this.fileName = "" + this.ddoc._id + "/" + this.funPath + ".js";
      code = normalizeFunctionDefinition(readPath(this.funPath, this.ddoc));
      sandbox = createSandbox(this.ddoc, this.funPath, this.log, this.overrides);
      try {
        this.fun = runInNewContext(code, sandbox, this.fileName);
      } catch (e) {

      }
      if (typeof this.fun !== 'function') {
        throw new NotAFunctionError(this.funPath, this.ddoc);
      }
    }
    CouchFunction.prototype.call = function() {
      var funArgs;
      funArgs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return this.fun.apply(null, funArgs);
    };
    return CouchFunction;
  }();
  readPath = function(propPath, obj) {
    var getSubObject;
    getSubObject = function(o, prop) {
      if (typeof o === 'object' && prop in o) {
        return o[prop];
      } else {
        throw new MissingPropPathError(propPath, obj);
      }
    };
    return propPath.split('.').reduce(getSubObject, obj);
  };
  normalizeFunctionDefinition = function(code) {
    var withoutSemicolon;
    withoutSemicolon = code.replace(/;$/, '');
    return "(" + withoutSemicolon + ");";
  };
  createSandbox = function(ddoc, propPath, logEntries, overrides) {
    var fun, name, sb;
    sb = {
      log: function(msg) {
        return logEntries.push(msg);
      },
      require: makeRequireFun(makeRefStack(ddoc, propPath))
    };
    for (name in overrides) {
      if (!__hasProp.call(overrides, name)) continue;
      fun = overrides[name];
      sb[name] = fun;
    }
    return sb;
  };
  makeRefStack = function(ddoc, funPath) {
    var stack;
    stack = [ddoc];
    funPath.split('.').forEach(function(prop) {
      var currObj;
      currObj = stack[stack.length - 1];
      return stack.push(currObj[prop]);
    });
    stack.pop();
    return stack;
  };
  makeRequireFun = function(refStack) {
    return function(moduleID) {
      var code, nextRefStack, sandbox, _ref;
      sandbox = {
        exports: {}
      };
      try {
        _ref = findModule(moduleID, refStack), nextRefStack = _ref[0], code = _ref[1];
        sandbox.require = makeRequireFun(nextRefStack);
        runInNewContext(code, sandbox, moduleID);
      } catch (e) {
        throw "Design doc require() error: " + e;
      }
      return sandbox.exports;
    };
  };
  findModule = function(moduleID, refStack) {
    var code, current, next, nextRefStack, part, _i, _len, _ref;
    code = null;
    nextRefStack = /^\.\.?\//.test(moduleID) ? refStack.slice() : refStack.slice(0, 1);
    _ref = moduleID.split('/');
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      part = _ref[_i];
      if (part === '..') {
        if (nextRefStack.length > 1) {
          nextRefStack.pop();
        }
      } else if (part !== '.') {
        current = nextRefStack[nextRefStack.length - 1];
        next = current[part];
        if (next == null) {
          throw "Couldn't find part '" + part + "' of '" + moduleID + "'";
        }
        nextRefStack.push(next);
      }
    }
    code = nextRefStack.pop();
    return [nextRefStack, code];
  };
  ViewMapFunction = function() {
    function ViewMapFunction(ddoc, viewName) {
      var emitted;
      this.ddoc = ddoc;
      this.viewName = viewName;
      this.emitted = emitted = [];
      ViewMapFunction.__super__.constructor.call(this, this.ddoc, "views." + viewName + ".map", {
        emit: function(k, v) {
          return emitted.push({
            key: k,
            value: v
          });
        }
      });
    }
    __extends(ViewMapFunction, CouchFunction);
    return ViewMapFunction;
  }();
  ViewReduceFunction = function() {
    function ViewReduceFunction(ddoc, viewName) {
      this.ddoc = ddoc;
      this.viewName = viewName;
      ViewReduceFunction.__super__.constructor.call(this, this.ddoc, "views." + viewName + ".reduce", {
        sum: function(values) {
          return values.reduce((function(total, n) {
            return total + n;
          }), 0);
        }
      });
    }
    __extends(ViewReduceFunction, CouchFunction);
    return ViewReduceFunction;
  }();
  exports.MissingPropPathError = MissingPropPathError = function() {
    function MissingPropPathError(funPath, ddoc) {
      this.funPath = funPath;
      this.ddoc = ddoc;
      this.message = "Property path '" + funPath + "' not found in design doc '" + this.ddoc._id + "'";
    }
    __extends(MissingPropPathError, Error);
    return MissingPropPathError;
  }();
  exports.NotAFunctionError = NotAFunctionError = function() {
    function NotAFunctionError(funPath, ddoc) {
      this.funPath = funPath;
      this.ddoc = ddoc;
      this.message = "Property path '" + funPath + "' in design doc '" + this.ddoc._id + "' does not evaluate to a function";
    }
    __extends(NotAFunctionError, Error);
    return NotAFunctionError;
  }();
}).call(this);
